@@ -1,8 +1,8 @@
+{-# LANGUAGE CPP                  #-}
+{-# LANGUAGE FlexibleInstances    #-}
+{-# LANGUAGE OverloadedStrings    #-}
+{-# LANGUAGE TypeOperators        #-}
 {-# LANGUAGE TypeSynonymInstances #-}
-{-# LANGUAGE TypeOperators #-}
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE FlexibleInstances #-}
-
 
 module Snap.Snaplet.Auth.Backends.JsonFile
   ( initJsonFileAuthManager
@@ -10,7 +10,6 @@ module Snap.Snaplet.Auth.Backends.JsonFile
   ) where
 
 
-import           Control.Applicative
 import           Control.Monad.State
 import           Control.Concurrent.STM
 import           Data.Aeson
@@ -26,6 +25,10 @@ import           Data.Time
 import           Web.ClientSession
 import           System.Directory
 
+#if !MIN_VERSION_base(4,8,0)
+import           Control.Applicative
+#endif
+
 import           Snap.Snaplet
 import           Snap.Snaplet.Auth.Types
 import           Snap.Snaplet.Auth.AuthManager
@@ -57,6 +60,7 @@ initJsonFileAuthManager s l db = do
                        , activeUser            = Nothing
                        , minPasswdLen          = asMinPasswdLen s
                        , rememberCookieName    = asRememberCookieName s
+                       , rememberCookieDomain  = Nothing
                        , rememberPeriod        = asRememberPeriod s
                        , siteKey               = key
                        , lockout               = asLockout s
@@ -85,11 +89,15 @@ mkJsonAuthMgr fp = do
 ------------------------------------------------------------------------------
 type UserIdCache = Map UserId AuthUser
 
+#if !MIN_VERSION_aeson(1,0,0)
+-- In aeson >= 1 these instances are not needed because we have
+-- derived ToJSONKey/FromJSONKey instances for UserId.
 instance ToJSON UserIdCache where
   toJSON m = toJSON $ HM.toList m
 
 instance FromJSON UserIdCache where
   parseJSON = fmap HM.fromList . parseJSON
+#endif
 
 ------------------------------------------------------------------------------
 type LoginUserCache = Map Text UserId
@@ -312,5 +320,3 @@ instance FromJSON UserCache where
       <*> v .: "tokenCache"
       <*> v .: "uidCounter"
   parseJSON _ = error "Unexpected JSON input"
-
-
