@@ -1,6 +1,6 @@
-{-# LANGUAGE BangPatterns               #-}
-{-# LANGUAGE OverloadedStrings          #-}
-{-# LANGUAGE TupleSections              #-}
+{-# LANGUAGE BangPatterns      #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE TupleSections     #-}
 
 module Snap.Snaplet.Internal.Initializer
   ( addPostInitHook
@@ -19,6 +19,7 @@ module Snap.Snaplet.Internal.Initializer
   , runSnaplet
   , combineConfig
   , serveSnaplet
+  , serveSnapletNoArgParsing
   , loadAppConfig
   , printInfo
   , getRoutes
@@ -26,36 +27,57 @@ module Snap.Snaplet.Internal.Initializer
   , modifyMaster
   ) where
 
-import           Prelude hiding (catch)
-import           Control.Concurrent.MVar
-import           Control.Exception (SomeException)
-import           Control.Lens
-import           Control.Monad
-import           Control.Monad.CatchIO hiding (Handler)
-import           Control.Monad.Reader
-import           Control.Monad.State
-import           Control.Monad.Trans.Either
-import           Control.Monad.Trans.Writer hiding (pass)
-import           Data.ByteString.Char8 (ByteString)
-import qualified Data.ByteString.Char8 as B
-import           Data.Configurator
-import qualified Data.Configurator.Types as C
-import           Data.IORef
-import           Data.Maybe
-import           Data.Text (Text)
-import qualified Data.Text as T
-import           Snap.Http.Server
-import           Snap.Core
-import           Snap.Util.GZip
-import           System.Directory
-import           System.Directory.Tree
-import           System.FilePath.Posix
-import           System.IO
-
-import           Snap.Snaplet.Config
-import qualified Snap.Snaplet.Internal.LensT as LT
+------------------------------------------------------------------------------
+import           Control.Applicative          ((<$>))
+import           Control.Concurrent.MVar      (MVar, modifyMVar_, newEmptyMVar,
+                                               putMVar, readMVar)
+import           Control.Exception.Lifted     (SomeException, catch, try)
+import           Control.Lens                 (ALens', cloneLens, over, set,
+                                               storing, (^#))
+import           Control.Monad                (Monad (..), join, liftM, unless,
+                                               when, (=<<))
+import           Control.Monad.Reader         (ask)
+import           Control.Monad.State          (get, modify)
+import           Control.Monad.Trans          (lift, liftIO)
+import           Control.Monad.Trans.Writer   hiding (pass)
+import           Data.ByteString.Char8        (ByteString)
+import qualified Data.ByteString.Char8        as B
+import           Data.Configurator            (Worth (..), addToConfig, empty,
+                                               loadGroups, subconfig)
+import qualified Data.Configurator.Types      as C
+import           Data.IORef                   (IORef, atomicModifyIORef,
+                                               newIORef, readIORef)
+import           Data.Maybe                   (Maybe (..), fromJust, fromMaybe,
+                                               isNothing)
+import           Data.Text                    (Text)
+import qualified Data.Text                    as T
+import           Prelude                      (Bool (..), Either (..), Eq (..),
+                                               String, concat, concatMap,
+                                               const, either,
+                                               error, filter, flip, fst, id,
+                                               map, not, show, ($), ($!), (++),
+                                               (.))
+import           Snap.Core                    (Snap, liftSnap, route)
+import           Snap.Http.Server             (Config, completeConfig,
+                                               getCompression, getErrorHandler,
+                                               getOther, getVerbose, httpServe)
+import           Snap.Util.GZip               (withCompression)
+import           System.Directory             (copyFile,
+                                               createDirectoryIfMissing,
+                                               doesDirectoryExist,
+                                               getCurrentDirectory)
+import           System.Directory.Tree        (DirTree (..), FileName, buildL,
+                                               dirTree, readDirectoryWith)
+import           System.FilePath.Posix        (dropFileName, makeRelative,
+                                               (</>))
+import           System.IO                    (FilePath, IO, hPutStrLn, stderr)
+------------------------------------------------------------------------------
+import           Snap.Snaplet.Config          (AppConfig, appEnvironment,
+                                               commandLineAppConfig)
 import qualified Snap.Snaplet.Internal.Lensed as L
+import qualified Snap.Snaplet.Internal.LensT  as LT
 import           Snap.Snaplet.Internal.Types
+------------------------------------------------------------------------------
 
 
 ------------------------------------------------------------------------------
@@ -96,11 +118,11 @@ getEnvironment = iGets _environment
 
 ------------------------------------------------------------------------------
 -- | Converts a plain hook into a Snaplet hook.
-toSnapletHook :: (v -> EitherT Text IO v)
-              -> (Snaplet v -> EitherT Text IO (Snaplet v))
+toSnapletHook :: (v -> IO (Either Text v))
+              -> (Snaplet v -> IO (Either Text (Snaplet v)))
 toSnapletHook f (Snaplet cfg  reset val) = do
     val' <- f val
-    return $! Snaplet cfg reset val'
+    return $! Snaplet cfg reset <$> val'
 
 
 ------------------------------------------------------------------------------
@@ -111,12 +133,12 @@ toSnapletHook f (Snaplet cfg  reset val) = do
 -- define its views.  The Heist snaplet provides the 'addTemplates' function
 -- which allows other snaplets to set up their own templates.  'addTemplates'
 -- is implemented using this function.
-addPostInitHook :: (v -> EitherT Text IO v)
+addPostInitHook :: (v -> IO (Either Text v))
                 -> Initializer b v ()
 addPostInitHook = addPostInitHook' . toSnapletHook
 
 
-addPostInitHook' :: (Snaplet v -> EitherT Text IO (Snaplet v))
+addPostInitHook' :: (Snaplet v -> IO (Either Text (Snaplet v)))
                  -> Initializer b v ()
 addPostInitHook' h = do
     h' <- upHook h
@@ -125,15 +147,15 @@ addPostInitHook' h = do
 
 ------------------------------------------------------------------------------
 -- | Variant of addPostInitHook for when you have things wrapped in a Snaplet.
-addPostInitHookBase :: (Snaplet b -> EitherT Text IO (Snaplet b))
+addPostInitHookBase :: (Snaplet b -> IO (Either Text (Snaplet b)))
                     -> Initializer b v ()
 addPostInitHookBase = Initializer . lift . tell . Hook
 
 
 ------------------------------------------------------------------------------
 -- | Helper function for transforming hooks.
-upHook :: (Snaplet v -> EitherT Text IO (Snaplet v))
-       -> Initializer b v (Snaplet b -> EitherT Text IO (Snaplet b))
+upHook :: (Snaplet v -> IO (Either Text (Snaplet v)))
+       -> Initializer b v (Snaplet b -> IO (Either Text (Snaplet b)))
 upHook h = Initializer $ do
     l <- ask
     return $ upHook' l h
@@ -141,10 +163,12 @@ upHook h = Initializer $ do
 
 ------------------------------------------------------------------------------
 -- | Helper function for transforming hooks.
-upHook' :: Monad m => ALens' b a -> (a -> m a) -> b -> m b
+upHook' :: Monad m => ALens' b a -> (a -> m (Either e a)) -> b -> m (Either e b)
 upHook' l h b = do
     v <- h (b ^# l)
-    return $ storing l v b
+    return $ case v of
+               Left e -> Left e
+               Right v' -> Right $ storing l v' b
 
 
 ------------------------------------------------------------------------------
@@ -256,7 +280,7 @@ mkSnaplet m = do
     l <- getLens
     let modifier = setInTop  . set (cloneLens l . snapletValue)
     return $ Snaplet cfg modifier res
-    
+
 
 
 ------------------------------------------------------------------------------
@@ -530,12 +554,12 @@ runInitializer' resetter env b@(Initializer i) cwd = do
     let cfg = SnapletConfig [] cwd Nothing "" empty [] Nothing reloader_
     logRef <- newIORef ""
 
-    let body = runEitherT $ do
-            ((res, s), (Hook hook)) <- lift $ runWriterT $ LT.runLensT i id $
+    let body = do
+            ((res, s), (Hook hook)) <- runWriterT $ LT.runLensT i id $
                 InitializerState True cleanupRef builtinHandlers id cfg logRef
                                  env resetter
             res' <- hook res
-            right (res', s)
+            return $ (,s) <$> res'
 
         handler e = do
             join $ readIORef cleanupRef
@@ -565,11 +589,11 @@ runSnaplet env (SnapletInit b) = do
     let resetter f = modifyMVar_ snapletMVar (return . f)
     eRes <- runInitializer resetter (fromMaybe "devel" env) b
     let go (siteSnaplet,is) = do
-        putMVar snapletMVar siteSnaplet
-        msgs <- liftIO $ readIORef $ _initMessages is
-        let handler = runBase (_hFilter is $ route $ _handlers is) snapletMVar
-        cleanupAction <- readIORef $ _cleanup is
-        return (msgs, handler, cleanupAction)
+            putMVar snapletMVar siteSnaplet
+            msgs <- liftIO $ readIORef $ _initMessages is
+            let handler = runBase (_hFilter is $ route $ _handlers is) snapletMVar
+            cleanupAction <- readIORef $ _cleanup is
+            return (msgs, handler, cleanupAction)
     either (error . ('\n':) . T.unpack) go eRes
 
 
@@ -601,13 +625,25 @@ serveSnaplet :: Config Snap AppConfig
                  -- ^ The snaplet initializer function.
              -> IO ()
 serveSnaplet startConfig initializer = do
-    config       <- commandLineAppConfig startConfig
+    config <- commandLineAppConfig startConfig
+    serveSnapletNoArgParsing config initializer
+
+------------------------------------------------------------------------------
+-- | Like 'serveSnaplet', but don't try to parse command-line arguments.
+serveSnapletNoArgParsing :: Config Snap AppConfig
+                 -- ^ The configuration of the server - you can usually pass a
+                 -- default 'Config' via
+                 -- 'Snap.Http.Server.Config.defaultConfig'.
+             -> SnapletInit b b
+                 -- ^ The snaplet initializer function.
+             -> IO ()
+serveSnapletNoArgParsing config initializer = do
     let env = appEnvironment =<< getOther config
     (msgs, handler, doCleanup) <- runSnaplet env initializer
 
     (conf, site) <- combineConfig config handler
     createDirectoryIfMissing False "log"
-    let serve = simpleHttpServe conf
+    let serve = httpServe conf
 
     when (loggingEnabled conf) $ liftIO $ hPutStrLn stderr $ T.unpack msgs
     _ <- try $ serve $ site
@@ -665,5 +701,5 @@ getCfg :: FileName -> DirTree b -> [b]
 getCfg cfg (Dir _ c) = map file $ filter (isCfg cfg) c
 getCfg _ _ = []
 
-    
+
 
